/*
  WMQ3 


  Best viewed with vim5, using ts=4
  
  This code was mainly put together by looking at the
  following programs:
  
  wmifs
  A very great app, displaying net traffic
  Source used:
  How do i create a dockapp with wmgeneral ?
  How do i handle Mouseactions ?
  ------------------------------------------------------------
  
  Author: Jens 'sober' Kirchheimer <sober@scriptgott.com>
  
  ----
  Thanks
  ----
*/
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>

#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "../wmgeneral/wmgeneral.h"
#include "../wmgeneral/misc.h"

#include "wmq3-master.xpm"
#include "wmq3_mask.xbm"


/***********/
/* Defines */
/***********/

/* Fill in the hardcoded actions */
#define LEFT_ACTION (NULL)
#define MIDDLE_ACTION (NULL)
#define RIGHT_ACTION (NULL)

#define WMQ3_VERSION "0.1.1"

/**********************/
/* External Variables */
/**********************/

extern	char **environ;

/********************/
/* Global Variables */
/********************/

char	*ProgName;
char	*active_interface = NULL;
int	TimerDivisor=60;
int	WaveForm=0;
int 	myStatus=0;

// serverstuff
char server[100][10];
char sip[100][25];
char playera[100][10];
char playerb[100][10];
char map[100][10];

// click call
char **clickcall;
int clickparms;

// ints
int currserver;
int scount;
int mem;


/***********************/
/* Function Prototypes */
/***********************/

void usage(void);
void printversion(void);
void DrawTime(int, int);
void DrawStats(int *, int, int, int, int);
int left(char *mleft, char *line,char limiter);
void strip(char *line);
int GetConfig();
void ButtonUp(int);
void ButtonDown(int);
void execute();
void wmq3_routine(int, char **);


/********/
/* Main */
/********/

int main(int argc, char *argv[]) 
{
  ProgName = argv[0];

  if (strlen(ProgName) >= 4)
    ProgName += (strlen(ProgName) - 4);

  if(GetConfig()<0)
    {
      printf("No servers in configfile\n");
      exit(0);
    }
  currserver=0;
  wmq3_routine(argc, argv);
  return 1;
}
void qstat()
{
  char call[256];
  FILE *read_fp;
  int counter;
  int chars_read;
  char buffer[10];
  char limiter;
  char Name[256];
  char Map[256];
  char Plp1[10];
  char Plp2[10];

  memset(buffer,0,sizeof(buffer));
  memset(call,0,sizeof(call));
  memset(Name,0,sizeof(Name));
  memset(Map,0,sizeof(Map));
  memset(Plp1,0,sizeof(Plp1));
  memset(Plp2,0,sizeof(Plp2));
  strcat(call,"qstat -R -raw \\; -q3s ");
  strcat(call,sip[currserver]);
  limiter = ';';
  
  counter=0;

  read_fp = popen(call,"r");
  if(read_fp!=NULL)
    {
      chars_read = fread(buffer,sizeof(char), 1, read_fp);
      while(chars_read>0)
	{
	  buffer[chars_read]='\0';
	  if(buffer[0]==';')
	    {
	      counter++;
	    }
	  else
	    {
	      if(counter==2)
		{
		  strcat(Name,buffer);
		}
	      if(counter==3)
		{
		  strcat(Map,buffer);
		}
	      if(counter==4)
		{
		  strcat(Plp1,buffer);
		}
	      if(counter==5)
		{
		  strcat(Plp2,buffer);
		}
	    }
	  chars_read = fread(buffer,sizeof(char), 1, read_fp);
	}
      pclose(read_fp);
    }
  // check for DOWN

  if(strlen(Name)>3)
    {
      memset(buffer,0,sizeof(buffer));
      for(counter=0;counter<4;counter++)
	buffer[counter]=Name[counter];
    }

  if(strcmp(buffer,"DOWN")==0)
    {
      memset(Map,0,sizeof(Map));
      strcat(Map,"DOWN");
    }


  // fill with SPACE
  for(counter=strlen(Map);counter<8;counter++)
    Map[counter]=' ';
  for(counter=strlen(Plp1);counter<4;counter++)
    Plp1[counter]=' ';
  for(counter=strlen(Plp2);counter<4;counter++)
    Plp2[counter]=' ';

  memset(map[currserver],0,sizeof(map[currserver]));
  memset(playera[currserver],0,sizeof(playera[currserver]));
  memset(playerb[currserver],0,sizeof(playerb[currserver]));
  strcat(map[currserver],Map);
  strcat(playera[currserver],Plp1);
  strcat(playerb[currserver],Plp2);
}
int StrPos(char *line,char limiter)
{
  int i;
  for(i=0;i<strlen(line);i++)
    {
      if(line[i]==limiter)
	return i;
    }
  return -1;
}
void drawText(int x, int y, char *text)
{
  char letters[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  char digits[]  = "0123456789 -";
  int i,lpos,mx,my;
  for(i=0;i<strlen(text);i++)
    {
      lpos=StrPos(letters,toupper(text[i]));
      if(lpos<0)
	{
	  // no letter
	  lpos=StrPos(digits,toupper(text[i]));
	  if(lpos<0)
	    {
	      // no digit
	      
	      // write a "-" instead	    
	      lpos=11;
	      my=65;
	    }
	  else
	    {
	      // draw digit
	      my=65;
	    }
	}
      else
	{
	  // print letter
	  my=75;
	}
      mx=1+(lpos*6);
      copyXPMArea(mx,my,6,7,x+(i*6),y);
    }
  RedrawWindow();
}

/*******************************************************************************\
|* wmifs_routine								*|
\*******************************************************************************/

char		*left_action = NULL;
char		*right_action = NULL;
char		*middle_action = NULL;

void wmq3_routine(int argc, char **argv) 
{
  int		i;
  XEvent		Event;
  int		but_stat = -1;
  
  long		starttime;
  long		curtime;
  long		nexttime;
  
  openXwindow(argc, argv, wmq3_master_xpm, wmq3_mask_bits, wmq3_mask_width, wmq3_mask_height);
  
  /* > Button */
  AddMouseRegion(0, 5, 5, 15, 16);
  AddMouseRegion(1, 49, 5, 58, 16);
  AddMouseRegion(2, 7, 5, 56, 16);
  AddMouseRegion(3, 5, 20, 58, 58);
  
  starttime = time(0);
  nexttime = starttime + 5;
  drawText(18,6,"  off");
  while (1) 
    {
      curtime = time(0);
      waitpid(0, NULL, WNOHANG);
      
      if (curtime >= nexttime) 
	{
	  nexttime+=5;
	  if(myStatus==1)
	    qstat();
	  RedrawWindow();
	}
      // print out stuff
      drawText(6,21,server[currserver]);
      drawText(11,34,map[currserver]);
      drawText(6,47,playerb[currserver]);
      drawText(35,47,playera[currserver]);


      while (XPending(display)) 
	{
	  XNextEvent(display, &Event);
	  switch (Event.type) 
	    {
	    case Expose:
	      RedrawWindow();
	      break;
	    case DestroyNotify:
	      XCloseDisplay(display);
	      exit(0);
	      break;
	    case ButtonPress:
	      i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
	      
	      but_stat = i;
	      break;
	    case ButtonRelease:
	      i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
	      
	      if (but_stat == i && but_stat >= 0) 
		{
		  switch (but_stat) 
		    {
		    case 0 :
		      /* re-read the table */
		      currserver--;
		      if(currserver<0)
			currserver=scount;
		        drawText(11,34,"        ");
		        drawText(6,47,"    ");
	                drawText(35,47,"    ");
		      break;
		    case 1:
		      currserver++;
		      if(currserver>scount)
			currserver=0;
		      drawText(11,34,"        ");
                      drawText(6,47,"    ");
		      drawText(35,47,"    ");
		      break;
		    case 2:
		      if(myStatus==0)
			{
			  drawText(18,6,"   on");
			  qstat();
			  myStatus=1;
			}
		      else
			{
			  drawText(18,6,"  off");
			  myStatus=0;
			}
		      break;
		    case 3:
		      execute();
		      break;
		      
		    }
		}
	      but_stat = -1;
	      RedrawWindow();
	      break;
	    }
	}
      
      usleep(50000L);
    }
}

/*******************************************************************************\
|* usage									*|
\*******************************************************************************/

void usage(void) {
  
  fprintf(stderr, "\nwmifs - programming: tijno, (de)bugging & design: warpstah, webhosting: bobby \n\n");
  fprintf(stderr, "usage:\n");
  fprintf(stderr, "\t-d <display name>\n");
  fprintf(stderr, "\t-h\tthis help screen\n");
  fprintf(stderr, "\t-i <interface name>\tdefault (as it appears in /proc/net/route)\n");
  fprintf(stderr, "\t-v\tprint the version number\n");
  fprintf(stderr, "\t-w\twaveform load\n");
  fprintf(stderr, "\n");
}

/*******************************************************************************\
|* printversion																   *|
\*******************************************************************************/

void printversion(void) {
  
  fprintf(stderr, "%s\n", WMQ3_VERSION);
}
/*******************************************************************************
 * execute clickcall
 *******************************************************************************/
void execute()
{
  pid_t pid;
  char path[256];
  int i;
  // replace ip if needed
  if(clickparms>0)
    {
      for(i=0;i<clickparms;i++)
	{
		printf("%s\n",clickcall[i]);
	  if(strcmp(clickcall[i],"$1")==0)
	    {
	      memset(clickcall[i],0, sizeof(clickcall[i]));
	      strcat(clickcall[i],sip[currserver]);
	      mem=i;
	    }
	}
      // quake needs to be in the directory where the exec is...
      // so ... get path:
      
      

      // is there a "/" ??
      if(StrPos(clickcall[0],'/')>-1)
	{
	  memset(path,0,sizeof(path));
	  strcat(path,clickcall[0]);
	  i=strlen(path);
	  
	  while(i>0)
	    {
	      i--;
	      if(path[i]=='/')
		{
		  path[i]='\0';
		  i=0;
		}
	    }
	}
      if(chdir(path))
	{
	  printf("Cannot change directory Drecksack!\n");
	  exit(1);
	}      
     	 
      if( (pid=fork())<0)
	{
	  printf("Error creating new process\n");
	  exit(1);
	}
      else if(pid== 0)
	{
	  execv(clickcall[0],clickcall);
	  exit(1);
	}
      else
	{
	  sleep(2);
	}
      memset(clickcall[mem],0,sizeof(clickcall[mem]));
      strcat(clickcall[mem],"$1");
    }
}
  /*******************************************************************************
 read config
******************************************************************************/
int GetConfig()
{
  FILE  *fh;
  char buffer[256];
  char temp[256];
  int i,n;
  scount=-1;

  clickcall = (char **)malloc(30 * sizeof(char*));

  // read configfile
  fh=fopen("/etc/wmq3rc","r");

  memset(buffer,0,sizeof(buffer));
  clickparms=-1;
  while(fgets(buffer,255,fh))
    {

      strip(buffer);
      // parse configfile
      if((strlen(buffer)==0) || (buffer[0]=='#'))
	{
	  //found command or empty line
	}
      else
	{
	  memset(temp,0,sizeof(temp));
	  left(temp,buffer,';');
	  if(strcmp(temp,"CALL")==0)
	    {
	      n=0;
	      clickparms=0;
	      clickcall[clickparms]=(char*)malloc(256*sizeof(char));
              memset(clickcall[clickparms],0,sizeof(clickcall[clickparms]));
	      for(i=0;i<strlen(buffer);i++)
		{
		  if(buffer[i]==' ')
		    {
		      clickparms++;
		      clickcall[clickparms]=(char*)malloc(256*sizeof(char));
		      memset(clickcall[clickparms],0,sizeof(clickcall[clickparms]));
		      n=0;
		    }
		  else
		    {
		      clickcall[clickparms][n]=buffer[i];
		      n++;
		    }
		}
	    }
	  else
	    {
	      scount++;
	      memset(server[scount],0,sizeof(server[scount]));
	      memset(sip[scount],0,sizeof(sip[scount]));
	      memset(map[scount],0,sizeof(map[scount]));
	      memset(playera[scount],0,sizeof(playera[scount]));
	      memset(playerb[scount],0,sizeof(playerb[scount]));
	      strcat(server[scount],temp);
	      strcat(sip[scount],buffer);
	      for(i=strlen(server[scount]);i<8;i++)
		server[scount][i]=' ';
	    }
	}
      memset(buffer,0,sizeof(buffer));
    }
  clickparms++;
  clickcall[clickparms]=(char*)malloc(256*sizeof(char));
  memset(clickcall[clickparms],0,sizeof(clickcall[clickparms]));
  clickcall[clickparms]=NULL;

  fclose(fh);
  return scount;
}
void strip(char *line)
{
  line[strlen(line)-1]='\0';
  return;
}
int left(char *mleft, char *line,char limiter)
{
  int i,m,t;
  char temp[800];
  
  memset(mleft,0,sizeof(mleft));
  memset(temp,0,sizeof(temp));
  t=0;
  m=0;
  for(i=0;i<strlen(line);i++)
    {
      if(line[i]==limiter)
	{
	  if(t==0)
	    {
	      m=0;
	      i++;
	    }
	  t=1;
	}
      if(t==0)
	mleft[m++]=line[i];
      else
	temp[m++]=line[i];
    }
  memset(line,0,sizeof(line));
  strcpy(line,temp);
  return 1;
}

